package gregapi.block.fluid;

import static gregapi.data.CS.F;
import static gregapi.data.CS.LIGHT_OPACITY_WATER;
import static gregapi.data.CS.SIDES_VERTICAL;
import static gregapi.data.CS.T;
import gregapi.data.LH;
import gregapi.util.UT;
import net.minecraft.block.material.Material;
import net.minecraft.client.renderer.texture.IIconRegister;
import net.minecraft.item.ItemBlock;
import net.minecraft.util.IIcon;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;
import net.minecraftforge.fluids.BlockFluidFinite;
import net.minecraftforge.fluids.Fluid;
import net.minecraftforge.fluids.FluidStack;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.relauncher.Side;
import cpw.mods.fml.relauncher.SideOnly;

/**
 * @author Gregorius Techneticies
 */
public class BlockBaseFluid extends BlockFluidFinite {
	public final String mNameInternal;
	public final int mFlammability;
	
	public BlockBaseFluid(String aNameInternal, Fluid aFluid, int aFlammability) {
		this(aNameInternal, aFluid, aFlammability, aFluid.getTemperature()>500?Material.lava:Material.water);
	}
	
	public BlockBaseFluid(String aNameInternal, Fluid aFluid, int aFlammability, Material aMaterial) {
		super(aFluid, aMaterial);
		setResistance(30);
		mFlammability = aFlammability;
		setBlockName(mNameInternal = aNameInternal);
		GameRegistry.registerBlock(this, ItemBlock.class, mNameInternal);
		LH.add(getUnlocalizedName()+".name", UT.Fluids.name(definedFluid, T));
	}
	
	@Override
	public FluidStack drain(World aWorld, int aX, int aY, int aZ, boolean aDoDrain) {
		// Forge royally fucked up again. You check for MetaData FIRST and do the set Block to Air SECOND, like I demonstrate here!!!
		FluidStack rFluid = UT.Fluids.make(getFluid(), (aWorld.getBlockMetadata(aX, aY, aZ) + 1) * 125);
		if (aDoDrain) aWorld.setBlockToAir(aX, aY, aZ);
		return rFluid;
	}
	
	@Override public final String getUnlocalizedName() {return UT.Fluids.name(definedFluid, F);}
	@Override public String getLocalizedName() {return UT.Fluids.name(definedFluid, T);}
	@Override public void registerBlockIcons(IIconRegister aIconRegister) {/**/}
    @Override public int getLightOpacity() {return LIGHT_OPACITY_WATER;}
	@Override public IIcon getIcon(int aSide, int aMeta) {return SIDES_VERTICAL[aSide]?definedFluid.getStillIcon():definedFluid.getFlowingIcon();}
	@Override @SideOnly(Side.CLIENT) public int getRenderColor(int aMeta) {return definedFluid.getColor();}
	@Override @SideOnly(Side.CLIENT) public int colorMultiplier(IBlockAccess aWorld, int aX, int aY, int aZ) {return definedFluid.getColor();}
	@Override public boolean canDisplace(IBlockAccess aWorld, int aX, int aY, int aZ) {return !aWorld.getBlock(aX, aY, aZ).getMaterial().isLiquid() && super.canDisplace(aWorld, aX, aY, aZ);}
	@Override public boolean displaceIfPossible(World aWorld, int aX, int aY, int aZ) {return !aWorld.getBlock(aX, aY, aZ).getMaterial().isLiquid() && super.displaceIfPossible(aWorld, aX, aY, aZ);}
	@Override public int getFireSpreadSpeed(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aDirection) {return mFlammability;}
	@Override public int getFlammability(IBlockAccess aWorld, int aX, int aY, int aZ, ForgeDirection aDirection) {return mFlammability;}
}