package gregapi.worldgen;

import gregapi.code.ArrayListNoNulls;
import gregapi.data.CS.ConfigsGT;
import gregapi.oredict.OreDictMaterial;

import java.util.ArrayList;
import java.util.Random;

import net.minecraft.world.World;
import net.minecraft.world.chunk.IChunkProvider;

/**
 * @author Gregorius Techneticies
 */
public class Worldgen_GT_Ore_Layer extends WorldgenObject {
	public static ArrayList<Worldgen_GT_Ore_Layer> sList = new ArrayListNoNulls<Worldgen_GT_Ore_Layer>();
	public static int sWeight = 0;
	public final short mMinY, mMaxY, mWeight, mDensity, mSize, mPrimaryMeta, mSecondaryMeta, mBetweenMeta, mSporadicMeta;
	public final boolean mOverworld, mNether, mEnd;
	
	public Worldgen_GT_Ore_Layer(String aName, boolean aDefault, int aMinY, int aMaxY, int aWeight, int aDensity, int aSize, boolean aOverworld, boolean aNether, boolean aEnd, OreDictMaterial aPrimary, OreDictMaterial aSecondary, OreDictMaterial aBetween, OreDictMaterial aSporadic) {
		super(aName, sList, aDefault);
		mOverworld			= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Overworld"												, aOverworld);
		mNether				= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Nether"												, aNether);
		mEnd				= ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "TheEnd"												, aEnd);
		mMinY				= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "MinHeight"					, aMinY);
		mMaxY				= (short)Math.max(mMinY+5, 	ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "MaxHeight"					, aMaxY));
		mWeight				= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "RandomWeight"				, aWeight);
		mDensity			= (short)					ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Density"						, aDensity);
		mSize				= (short)Math.max(1,		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "Size"						, aSize));
		mPrimaryMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OrePrimaryLayer"				, aPrimary.mNameInternal)).mID;
		mSecondaryMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSecondaryLayer"			, aSecondary.mNameInternal)).mID;
		mBetweenMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSporadiclyInbetween"		, aBetween.mNameInternal)).mID;
		mSporadicMeta		= OreDictMaterial.get(		ConfigsGT.WORLDGEN.get("worldgen."+mWorldGenName, "OreSporaticlyAround"			, aSporadic.mNameInternal)).mID;
		if (mEnabled) sWeight += mWeight;
	}
	
	public boolean executeWorldgen(World aWorld, Random aRandom, String aBiome, int aDimensionType, int aChunkX, int aChunkZ, int aOriginChunkX, int aOriginChunkZ, IChunkProvider aChunkGenerator, IChunkProvider aChunkProvider) {
		if (aWorld.provider.dimensionId == -2147483648 || !isGenerationAllowed(aWorld, aDimensionType, (aDimensionType == -1 && mNether) || (aDimensionType == 0 && mOverworld) || (aDimensionType == +1 && mEnd) ? aDimensionType : ~aDimensionType)) return false;
		
		int tMinY = mMinY + aRandom.nextInt(mMaxY - mMinY - 5);
		
		for (int cX=aOriginChunkX-aRandom.nextInt(mSize), eX=aOriginChunkX+16+aRandom.nextInt(mSize), tX=Math.max(aChunkX, cX); tX<=Math.min(aChunkX+15, eX); tX++) {
			for (int cZ=aOriginChunkZ-aRandom.nextInt(mSize), eZ=aOriginChunkZ+16+aRandom.nextInt(mSize), tZ=Math.max(aChunkZ, cZ); tZ<=Math.min(aChunkZ+15, eZ); tZ++) {
				if (mSecondaryMeta >= 0) for (int i=tMinY-1; i<tMinY+2; i++) {
					if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
						Worldgenerator.setOreBlock(aWorld, tX, i, tZ, mSecondaryMeta);
					}
				}
				if (mBetweenMeta >= 0) if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
					Worldgenerator.setOreBlock(aWorld, tX, tMinY+2+aRandom.nextInt(2), tZ, mBetweenMeta);
				}
				if (mPrimaryMeta >= 0) for (int i=tMinY+3; i<tMinY+6; i++) {
					if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
						Worldgenerator.setOreBlock(aWorld, tX, i, tZ, mPrimaryMeta);
					}
				}
				if (mSporadicMeta >= 0) if (aRandom.nextInt(Math.max(1, Math.max(Math.abs(cZ-tZ), Math.abs(eZ-tZ)) / mDensity)) == 0 || aRandom.nextInt(Math.max(1, Math.max(Math.abs(cX-tX), Math.abs(eX-tX)) / mDensity)) == 0) {
					Worldgenerator.setOreBlock(aWorld, tX, tMinY-1+aRandom.nextInt(7), tZ, mSporadicMeta);
				}
			}
		}
		return true;
	}
}